(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global['hofs/defaultHeaders'] = {}));
}(this, function (exports) { 'use strict';

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function ownKeys(object, enumerableOnly) {
    var keys = Object.keys(object);

    if (Object.getOwnPropertySymbols) {
      keys.push.apply(keys, Object.getOwnPropertySymbols(object));
    }

    if (enumerableOnly) keys = keys.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    return keys;
  }

  function _objectSpread2(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i] != null ? arguments[i] : {};

      if (i % 2) {
        ownKeys(source, true).forEach(function (key) {
          _defineProperty(target, key, source[key]);
        });
      } else if (Object.getOwnPropertyDescriptors) {
        Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
      } else {
        ownKeys(source).forEach(function (key) {
          Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
        });
      }
    }

    return target;
  }

  var headers = {};
  var addDefaultHeader = function addDefaultHeader(key, value) {
    headers[key] = value;
  };
  var removeDefaultHeader = function removeDefaultHeader(key) {
    delete headers[key];
  };
  /**
   * Add default headers]
   * @example
   * import {addDefaultHeader, defaultHeaders} from 'higher-order-fetch/hofs/defaultHeaders'
   *
   * const fetch = defaultHeaders({
   *  "Content-Type": "application/json",
   * })(window.fetch)
   *
   * // called after login success:
   * addDefaultHeader('Authorization', 'Basic Ym9zY236Ym9zY28=')
   *
   * // below request should have 2 headers added from above:
   * fetch('http://example.com/data')
   * .then(response => response.json())
   * .then(data => { ... })
   * @param {object} [defaultHeaders]
   * @returns {function} a fetch-like function
   */

  var defaultHeaders = function defaultHeaders() {
    var defaultHeaders = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return function (fetch) {
      return function (input) {
        var init = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        return fetch(input, _objectSpread2({}, init, {
          headers: mergeWithDefaultHeaders(init.headers, _objectSpread2({}, defaultHeaders, {}, headers))
        }));
      };
    };
  };

  function mergeWithDefaultHeaders() {
    var initHeaders = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var defaultHeaders = arguments.length > 1 ? arguments[1] : undefined;
    var headerObj;

    if (initHeaders instanceof Headers) {
      headerObj = initHeaders.entries() //convert array of entries into object:
      .reduce(function (acc, val) {
        return _objectSpread2({}, acc, _defineProperty({}, val[0], val[1]));
      }, {});
    } else {
      headerObj = initHeaders;
    }

    return _objectSpread2({}, defaultHeaders, {}, headerObj);
  }

  exports.addDefaultHeader = addDefaultHeader;
  exports.defaultHeaders = defaultHeaders;
  exports.removeDefaultHeader = removeDefaultHeader;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
