(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global['hofs/jsonResponse'] = {}));
}(this, function (exports) { 'use strict';

  function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
      var info = gen[key](arg);
      var value = info.value;
    } catch (error) {
      reject(error);
      return;
    }

    if (info.done) {
      resolve(value);
    } else {
      Promise.resolve(value).then(_next, _throw);
    }
  }

  function _asyncToGenerator(fn) {
    return function () {
      var self = this,
          args = arguments;
      return new Promise(function (resolve, reject) {
        var gen = fn.apply(self, args);

        function _next(value) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
        }

        function _throw(err) {
          asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
        }

        _next(undefined);
      });
    };
  }

  /**
   * Implicitly parse response to json. The fetch created by this will resolve an array of [jsonData, statusCode]
   * @example
   * import {jsonResponse} from 'higher-order-fetch/hofs/jsonResponse'
   * const fetch = jsonResponse()(window.fetch)
   *
   * fetch('http://example.com/data')
   * .then(([data, status]) => {// no need to call `response.json()` here
   * // `data` is a JSON object only if fetch data success with JSON data, otherwise, it is either a Response or an Error.
   * // `status` is either HTTP Status Code or `undefined` if fetch fail.
   * })
   *
   * @param {boolean} suppressWarning If true, no error will be logged when request error or parse JSON fail.
   * @returns {function} a fetch-like function
   */
  var jsonResponse = function jsonResponse() {
    var suppressWarning = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    return function (fetch) {
      return (
        /*#__PURE__*/
        _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee() {
          var response,
              json,
              _args = arguments;
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  _context.prev = 0;
                  _context.next = 3;
                  return fetch.apply(void 0, _args);

                case 3:
                  response = _context.sent;
                  _context.next = 6;
                  return response.json();

                case 6:
                  json = _context.sent;
                  return _context.abrupt("return", [json, response.status]);

                case 10:
                  _context.prev = 10;
                  _context.t0 = _context["catch"](0);
                  !suppressWarning && console.warn(_context.t0);
                  return _context.abrupt("return", response ? [response, response.status] : [_context.t0, undefined]);

                case 14:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, null, [[0, 10]]);
        }))
      );
    };
  };

  exports.jsonResponse = jsonResponse;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
